import datetime
import os.path
import unittest
from urllib import pathname2url

import pytest

from upyun import const, UpYun


class UpYunTestCase(unittest.TestCase):
    #: File type bucket
    BUCKET_FILE = ''

    #: Image type bucket
    BUCKET_IMAGE = ''

    #: Username
    USERNAME = ''

    #: Password
    PASSWD = ''

    #: Predefined thumbnail version
    THUMB_VERSION = ''

    ASSET = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'asset')
    LOCAL_PATH_TXT_FILE = os.path.join(ASSET, 'upyun-test.txt')
    LOCAL_PATH_IMG_FILE = os.path.join(ASSET, 'upyun-test.gif')
    REMOTE_DIR = '/tmp/upyun-test'
    REMOTE_PATH_TXT_FILE = pathname2url(
            os.path.join(REMOTE_DIR, 'upyun-test.txt'))
    REMOTE_PATH_IMG_FILE = pathname2url(
            os.path.join(REMOTE_DIR, 'upyun-test.gif'))

    def setUp(self):
        self.client_file = UpYun(self.BUCKET_FILE,
                (self.USERNAME, self.PASSWD), const.SPACE_TYPE_FILE)
        self.client_image = UpYun(self.BUCKET_IMAGE,
                (self.USERNAME, self.PASSWD), const.SPACE_TYPE_IMAGE)
        self.test_file_txt = open(self.LOCAL_PATH_TXT_FILE)
        self.test_file_img = open(self.LOCAL_PATH_IMG_FILE, 'rb')

    def tearDown(self):
        self.test_file_txt.close()
        self.test_file_img.close()
        self._delete(self.REMOTE_PATH_TXT_FILE, self.client_file)
        self._delete(self.REMOTE_PATH_IMG_FILE, self.client_file)
        self._delete(self.REMOTE_PATH_IMG_FILE, self.client_image)
        self._delete(self.REMOTE_DIR, self.client_image)
        self._delete(self.REMOTE_DIR, self.client_file)

    def _put_file(self):
        return self.client_file.put(
                self.REMOTE_PATH_TXT_FILE, self.test_file_txt)

    def _put_image(self, client=None):
        client = client or self.client_image
        return client.put(self.REMOTE_PATH_IMG_FILE, self.test_file_img)

    def _mkdir(self, mk_parent=True, client=None):
        client = client or self.client_file
        return client.mkdir(self.REMOTE_DIR, mk_parent)

    def _delete(self, path, client):
        return client.delete(path)

    def test_put_file_space_file(self):
        resp = self._put_file()
        assert resp.success, resp.error

    def test_put_file_space_image(self):
        resp = self._put_image(self.client_file)
        assert resp.success, resp.error

    def test_put_image_space_image(self):
        resp = self._put_image(self.client_image)
        assert resp.success, resp.error
        assert isinstance(resp.frames, int)
        assert isinstance(resp.height, int)
        assert resp.height > 0
        assert isinstance(resp.width, int)
        assert resp.width > 0

    def test_put_thumbnail_version(self):
        resp = self.client_image.put_thumbnail(self.REMOTE_PATH_IMG_FILE,
                self.test_file_img, self.THUMB_VERSION)
        assert resp.success, resp.error

    def test_put_thumbnail_version_modified(self):
        resp = self.client_image.put_thumbnail(self.REMOTE_PATH_IMG_FILE,
                self.test_file_img, self.THUMB_VERSION,
                const.THUMB_TYPE_FIX_MAX, (10,), 100, True)
        assert resp.success, resp.error

    def test_put_thumbnail_custom(self):
        resp = self.client_image.put_thumbnail(self.REMOTE_PATH_IMG_FILE,
                self.test_file_img, ttype=const.THUMB_TYPE_FIX_MAX, res=(10,),
                quality=100, sharpen=True)
        assert resp.success, resp.error

    def test_get_text_file(self):
        self._put_file()
        resp = self.client_file.get(self.REMOTE_PATH_TXT_FILE)
        assert resp.success, resp.error
        self.test_file_txt.seek(0)
        assert resp.data == self.test_file_txt.read()

    def test_get_binary_file(self):
        client = self.client_image
        self._put_image(client)
        resp = client.get(self.REMOTE_PATH_IMG_FILE)
        assert resp.success, resp.error
        self.test_file_img.seek(0)
        assert resp.data == self.test_file_img.read()

    def test_ls(self):
        client = self.client_file
        self._put_file()
        self._put_image(client)
        resp = client.ls(self.REMOTE_DIR)
        assert resp.success, resp.error
        remote_file_paths = map(lambda f: f.path, resp.files.itervalues())
        assert self.REMOTE_PATH_TXT_FILE in remote_file_paths
        assert self.REMOTE_PATH_IMG_FILE in remote_file_paths

    def test_mkdir(self):
        resp = self._mkdir()
        assert resp.success, resp.error

    def test_usage(self):
        resp = self.client_file.usage()
        assert resp.success, resp.error
        assert isinstance(resp.usage, int)
        resp = self.client_image.usage()
        assert resp.success, resp.error
        assert isinstance(resp.usage, int)

    def test_info(self):
        def _assert(resp):
            assert resp.success, resp.error
            assert isinstance(resp.size, int)
            assert isinstance(resp.date, datetime.datetime)
            assert resp.type == const.FILE_TYPE_FILE

        self._put_file()
        resp = self.client_file.info(self.REMOTE_PATH_TXT_FILE)
        _assert(resp)
        self._put_image()
        resp = self.client_image.info(self.REMOTE_PATH_IMG_FILE)
        _assert(resp)

    def test_delete(self):
        self._put_file()
        resp = self.client_file.delete(self.REMOTE_PATH_TXT_FILE)
        assert resp.success, resp.error
        self._put_image()
        resp = self.client_image.delete(self.REMOTE_PATH_IMG_FILE)
        assert resp.success, resp.error

if __name__ == '__main__':
    unittest.main()



import hashlib
import os.path
from urllib import pathname2url
from urlparse import urljoin

import requests
from requests.auth import AuthBase

from . import const, response
from .auth import UpYunDigestAuthentication

__title__ = 'pyupyun'
__version__ = '0.1'
__author__ = 'Kane Dou'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2013 Kane Dou'


class UpYun(object):
    """Feature complete UpYun REST client

    :param str bucket: Your bucket
    :param auth: ``(username, passwd)`` pair or any object derived from
                 :class:`requests.auth.AuthBase`
    :param stype: The space type, :const:`~const.SPACE_TYPE_FILE` or
                  :const:`~const.SPACE_TYPE_IMAGE`
    :param api_host: API host to use, see :ref:`API Hosts <api-hosts>`
    :param str domain: Your custom domain
    :param bool ssl: Whether to use SSL

    Usage::

        client = UpYun('test', ('user', 'pass'), const.SPACE_TYPE_FILE)
        client.put('/test.txt', open('/tmp/test.txt'))
    """
    def __init__(self, bucket, auth, stype, api_host=const.API_HOST_AUTO,
            domain=None, ssl=False):
        self.bucket = bucket
        self.stype = stype
        self.api_host = api_host
        proto = 'https://' if ssl else 'http://'
        self._base_url = proto + const.UPAIYUN_API_HOSTS[self.api_host]
        self.domain = domain or (const.BUCKET_DOMAIN % bucket)
        self._bucket_base_url = proto + self.domain

        #: The :class:`requests.Session` object to use for the API requests
        self.session = self._prepare_session(auth, ssl)

    @property
    def api_host(self):
        return self._api_host

    @api_host.setter
    def api_host(self, api_host):
        if api_host in const.UPAIYUN_API_HOSTS:
            self._api_host = api_host
        else:
            raise Exception('api_host: invalid api host')

    @property
    def stype(self):
        return self._stype

    @stype.setter
    def stype(self, stype):
        if stype in [const.SPACE_TYPE_FILE, const.SPACE_TYPE_IMAGE]:
            self._stype = stype
        else:
            raise Exception('stype: invalid space type')

    def _prepare_session(self, auth, ssl):
        if isinstance(auth, AuthBase):
            self._auth = auth
        else:
            self._auth = auth if ssl else UpYunDigestAuthentication(*auth)
        s = requests.Session()
        s.auth = self._auth
        return s

    def _get_file_url(self, path):
        return urljoin(self._bucket_base_url, path)

    def _get_url(self, path):
        return urljoin(self._base_url,
                pathname2url(os.path.join(self.bucket, path.lstrip('/'))))

    def _get_data(self, fileo):
        try:
            data = fileo.read()
        except AttributeError:
            data = fileo
        return data

    def _digest(self, data):
        return hashlib.md5(data).hexdigest()

    def _prepare_put_request(self, path, fileo, mkdir, mimetype, secret,
            verify, headers=None):
        """Prepaer the put request"""
        url = self._get_url(path)
        data = self._get_data(fileo)
        headers = headers or {}
        req_headers = {}

        if mkdir:
            req_headers[const.HEADER_MKDIR] = 'true'
        if verify:
            req_headers[const.HEADER_MD5] = self._digest(data)
        if mimetype:
            req_headers['Content-Type'] = mimetype
        if secret:
            req_headers[const.HEADER_SECRET] = secret

        req_headers.update(headers)
        return url, data, req_headers

    def put(self, path, fileo, mkdir=True, mimetype=None, secret=None,
            verify=True, headers=None):
        """Put an file onto the server

        :param path: File path on the server
        :param fileo: File like object of the file to upload
        :param mkdir: Whether to make the parent folder if not existed
        :param mimetype: Mime-type of the file, used by server to determine
                         the extension of the file
        :param secret: Secret for user to later access the file uploaded
        :param verify: Whether to verify the file integrity using md5 hashing
        :param headers: Additional headers
        :rtype: :class:`~response.Response` or
                :class:`~response.PutImageResponse`
        """
        url, data, headers = self._prepare_put_request(path, fileo, mkdir,
                mimetype, secret, verify, headers or {})
        resp = self.session.put(url=url, data=data, headers=headers)
        if self.stype == const.SPACE_TYPE_IMAGE:
            return response.PutImageResponse(resp, self._get_file_url(path))
        return response.Response(resp, self._get_file_url(path))

    def put_thumbnail(self, path, fileo, version=None, ttype=None, res=None,
            quality=None, sharpen=None, **kwargs):
        """Put an image as a thumbnail on the server, the original image
        will not be uploaded

        :param path: Thumbnail path on the server
        :param fileo: File like object of the image
        :param version: Predefined thumbnail version name
        :param ttype: Thumbnail type, see :const:`~const.THUMB_TYPES`
        :param res: Image resolution, format: (width, height)
        :param quality: Image quality, default: 90
        :param sharpen: Whether to sharpen the image
        :type res: tuple
        :type sharpen: bool
        :rtype: :class:`~response.PutImageResponse`
        """
        if not (version or (ttype and res)):
            raise Exception(
                    'put: needs either version or thumbnail type, or both')
        headers = {}
        if version:
            if self.stype != const.SPACE_TYPE_IMAGE:
                raise Exception('put: need space type image')
            headers[const.HEADER_THUMB_VERSION] = version
        if ttype:
            if ttype not in const.THUMB_TYPES:
                raise Exception('put: invalid thumbnail type')
            headers[const.HEADER_THUMB_TYPE] = const.THUMB_TYPES[ttype]
            if res:
                if ttype in (const.THUMB_TYPE_FIX_BOTH,
                        const.THUMB_TYPE_FIX_WIDTH_OR_HEIGHT):
                    res_val = res[0] + 'x' + res[1]
                else:
                    res_val = res[0]
                headers[const.HEADER_THUMB_VALUE] = res_val
        if quality:
            headers[const.HEADER_THUMB_QUALITY] = int(quality)
        if sharpen is not None:
            headers[const.HEADER_THUMB_UNSHARP] = str(not sharpen).lower()

        uheaders = kwargs.pop('headers', {})
        headers.update(uheaders)

        return self.put(path, fileo, headers=headers, **kwargs)

    def get(self, path):
        """Get a file

        :param path: Path of the file to retrieve
        :rtype: :class:`~response.GetResponse`
        """
        resp = self.session.get(self._get_url(path))
        return response.GetResponse(resp, self._get_file_url(path))

    def delete(self, path):
        """Delete a file or an empty folder

        :param path: Path of the file or folder to delete
        :rtype: :class:`~response.Response`
        """
        resp = self.session.delete(self._get_url(path))
        return response.Response(resp, None)

    def mkdir(self, path, mk_parent=True):
        """Create a folder on server

        :param path: Folder path
        :param mk_parent: Whether to create the parent folder if not existed
        :rtype: :class:`~response.Response`
        """
        url = self._get_url(path)
        headers = {}
        headers[const.HEADER_FOLDER] = 'create'
        if mk_parent:
            headers[const.HEADER_MKDIR] = 'true'
        resp = self.session.post(url, headers=headers)
        return response.Response(resp, self._get_file_url(path))

    def ls(self, path):
        """List contents of a folder

        :param path: Path to the folder
        :rtype: :class:`~response.LsResponse`
        """
        resp = self.session.get(self._get_url(path))
        return response.LsResponse(resp, self._get_file_url(path))

    def usage(self):
        """Retrieve the space usage info

        :rtype: :class:`~response.UsageResponse`
        """
        resp = self.session.get(self._get_url(''), params='usage')
        return response.UsageResponse(resp, None)

    def info(self, path):
        """Retrieve file info

        :rtype: :class:`~response.InfoResponse`
        """
        resp = self.session.head(self._get_url(path))
        return response.InfoResponse(resp, self._get_file_url(path))



import datetime
import hashlib

from requests.auth import AuthBase


class UpYunDigestAuthentication(AuthBase):
    """UpYun signature digest authentication implemented for `requests`

    :param str user: Username
    :param str passwd: Password
    """
    AUTH_STR = 'UpYun'

    def __init__(self, user, passwd):
        self.user = user
        self.passwd_digest = hashlib.md5(passwd).hexdigest()

    def _get_content_length(self, r):
        if r.headers['content-length'] is not None:
            return int(r.headers['content-length'])
        try:
            return len(r.body)
        except (AttributeError, TypeError):
            pass
        try:
            return len(r.data)
        except (AttributeError, TypeError):
            pass
        return 0

    def __call__(self, r):
        datetime_str = datetime.datetime.utcnow().strftime(
                '%a, %d %b %Y %H:%M:%S GMT')
        sign_base = '&'.join((r.method, r.path_url, datetime_str,
            str(self._get_content_length(r)), self.passwd_digest))
        sign = hashlib.md5(sign_base).hexdigest()
        r.headers['Date'] = datetime_str
        r.headers['Authorization'] = \
                self.AUTH_STR + ' ' + self.user + ':' + sign
        return r



#!/usr/bin/env python
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='pyupyun',
    version='0.1',
    description='Feature complete upyun REST client',
    long_description=open('README.rst').read(),
    author='Kane Dou',
    author_email='douqilong@gmail.com',
    url='https://github.com/kols/upyun',
    packages=['upyun'],
    package_data={'': ['LICENSE']},
    package_dir={'upyun': 'upyun'},
    include_package_data=True,
    install_requires=['requests'],
    license=open('LICENSE').read(),
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ),
)



API_HOST_TELECOM = 1
API_HOST_NETCOM = 2
API_HOST_RAILCOM = 3
API_HOST_AUTO = 4
UPAIYUN_API_HOSTS = {
        API_HOST_AUTO: 'v0.api.upyun.com',
        API_HOST_TELECOM: 'v1.api.upyun.com',
        API_HOST_NETCOM: 'v2.api.upyun.com',
        API_HOST_RAILCOM: 'v3.api.upyun.com',
        }

SPACE_TYPE_FILE = 1
SPACE_TYPE_IMAGE = 2

BUCKET_DOMAIN = '%s.b0.upaiyun.com'

HEADER_UPYUN_PREFIX = 'x-upyun-'
HEADER_THUMB_PREFIX = 'x-gmkerl-'
HEADER_THUMB_VERSION = HEADER_THUMB_PREFIX + 'thumbnail'
HEADER_THUMB_TYPE = HEADER_THUMB_PREFIX + 'type'
HEADER_THUMB_VALUE = HEADER_THUMB_PREFIX + 'value'
HEADER_THUMB_QUALITY = HEADER_THUMB_PREFIX + 'quality'
HEADER_THUMB_UNSHARP = HEADER_THUMB_PREFIX + 'unsharp'
HEADER_MKDIR = 'Mkdir'
HEADER_MD5 = 'Content-MD5'
HEADER_SECRET = 'Content-Secret'
HEADER_FOLDER = 'Folder'

THUMB_TYPE_FIX_MAX = 1
THUMB_TYPE_FIX_WIDTH_OR_HEIGHT = 2
THUMB_TYPE_FIX_WIDTH = 3
THUMB_TYPE_FIX_HEIGHT = 4
THUMB_TYPE_SQUARE = 5
THUMB_TYPE_FIX_BOTH = 6
THUMB_TYPES = {
        THUMB_TYPE_FIX_MAX: 'fix_max',
        THUMB_TYPE_FIX_WIDTH_OR_HEIGHT: 'fix_width_or_height',
        THUMB_TYPE_FIX_WIDTH: 'fix_width',
        THUMB_TYPE_FIX_HEIGHT: 'fix_height',
        THUMB_TYPE_SQUARE: 'square',
        THUMB_TYPE_FIX_BOTH: 'fix_both',
        }

FILE_TYPE_FILE = 1
FILE_TYPE_FOLDER = 2



from collections import namedtuple
from datetime import datetime
import os.path
import urllib
import urlparse

import requests

from upyun import const


class ResponseBase(object):
    """A response of successfully uploading image, contains extra info
    of the uploaded image

    :param response: Response from UpYun
    :type response: :class:`requests.Response`
    :param str url: URL of the file on the UpYun
    """
    def __init__(self, response, url):
        #: Raw :class:`requests.Response` from UpYun
        self.response = response

        #: URL of the file on the UpYun
        self.url = url

        #: Error of the request, a :class:`tuple` in the form of
        #: ``(<status code>, <error message>)``
        self.error = self._populate_error()

    def _get_header_with_prefix(self, name):
        return self.response.headers.get(const.HEADER_UPYUN_PREFIX + name)

    def _populate_error(self):
        if not self.success:
            return (self.response.status_code, self.response.content.strip())

    @property
    def success(self):
        """Whether the API request is successful"""
        return self.response.status_code == requests.codes.ok


class ImageInfoMixin(object):
    @property
    def width(self):
        """
        Width of the image

        :rtype: :class:`int` or :class:`None`
        """
        w = self._get_header_with_prefix('width')
        return int(w) if w else None

    @property
    def height(self):
        """
        Height of the image

        :rtype: :class:`int` or :class:`None`
        """
        h = self._get_header_with_prefix('height')
        return int(h) if h else None

    @property
    def frames(self):
        """
        Frames of the image

        :rtype: :class:`int` or :class:`None`
        """
        f = self._get_header_with_prefix('frames')
        return int(f) if f else None


class FileInfoMixin(object):
    @property
    def date(self):
        """File created date

        :rtype: :class:`datetime.datetime` or :class:`None`
        """
        d = self._get_header_with_prefix('file-date')
        return datetime.utcfromtimestamp(int(d)) if d else None

    @property
    def size(self):
        """File size

        :rtype: :class:`int` or :class:`None`
        """
        s = self._get_header_with_prefix('file-size')
        return int(s) if s else None


class UsageMixin(object):
    @property
    def usage(self):
        """Usage of the space in bytes

        :rtype: :class:`int` or :class:`None`
        """
        return int(self.response.text) if self.response.text else None


class FileTypeMixin(object):
    @property
    def type(self):
        """File type of the queried path

        :const:`~upyun.const.FILE_TYPE_FILE` or
        :const:`~upyun.const.FILE_TYPE_FOLDER`
        """
        ft = self._get_header_with_prefix('file-type')
        if ft:
            ft = ft.lower()
            if ft == 'file':
                return const.FILE_TYPE_FILE
            elif ft == 'folder':
                return const.FILE_TYPE_FOLDER


class GetMixin(object):
    @property
    def data(self):
        """Data of the downloaded file"""
        return self.response.content


class LsMixin(object):
    TYPE_FILE_STR = 'N'
    TYPE_FOLDER_STR = 'F'

    FileInfo = namedtuple('FileInfo',
            ['name', 'path', 'url', 'type', 'size', 'mtime'])

    def _parse_response(self):
        self._files = {}
        self._folders = {}

        for l in self.response.content.strip().split("\n"):
            name, type, size, mtime = l.strip().split()

            type = type.upper()
            url_parsed = urlparse.urlparse(self.url)
            folder_path = url_parsed.path

            path = urllib.pathname2url(os.path.join(folder_path, name))
            url = urlparse.urljoin(
                    (url_parsed.scheme + '://' + url_parsed.netloc), path)
            mtime = datetime.utcfromtimestamp(int(mtime))

            if type == self.TYPE_FILE_STR:
                f = self.FileInfo(name=name, path=path, url=url,
                        type=const.FILE_TYPE_FILE, size=int(size), mtime=mtime)
                self._files[name] = f
            elif type == self.TYPE_FOLDER_STR:
                f = self.FileInfo(name=name, path=path, url=url,
                        type=const.FILE_TYPE_FOLDER, size=int(size), mtime=mtime)
                self._folders[name] = f

    @property
    def files(self):
        """Files in the directory

        Key is the file name, value is
        :class:`~upyun.response.LsMixin.FileInfo`

        :rtype: :class:`dict`
        """
        if not hasattr(self, '_files'):
            self._parse_response()
        return self._files

    @property
    def folders(self):
        """Folders in the directory

        Key is the folder name, value is
        :class:`~upyun.response.LsMixin.FileInfo`

        :rtype: :class:`dict`
        """
        if not hasattr(self, '_folders'):
            self._parse_response()
        return self._folders

    @property
    def stuffs(self):
        """All the stuffs in the directory

        Key is file or folder name, value is
        :class:`~upyun.response.LsMixin.FileInfo`

        :rtype: :class:`dict`
        """
        stuffs = {}
        stuffs.update(self.files)
        stuffs.update(self.folders)
        return stuff


class PutImageResponse(ResponseBase, ImageInfoMixin, FileTypeMixin):
    pass


class InfoResponse(ResponseBase, FileInfoMixin, FileTypeMixin):
    pass


class UsageResponse(ResponseBase, UsageMixin):
    pass


class GetResponse(ResponseBase, GetMixin):
    pass


class LsResponse(ResponseBase, LsMixin):
    pass


class Response(ResponseBase):
    pass
